#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.c"

#define EPSILON 1.0E-12

int main(int argc, char *argv[])
{
	FILE *fp_in;
	FILE *fp_out;
	int i, j;
	int 	analsize;
	int pd;
	matrix 	sim, evec, evec_work;
	vector	eval, eval_work, l_mean;
	vector cont, cumcont;
	char 	buffer[256];
	double	t_mean, sum1, sum2;

	/* show title */
	printf("pco 2.0 - for Win32 (c) 2005 Hiroyoshi IWATA\n");
	printf("Software for principal co-ordinate analysis\n\n");

	/* get input file name */
	if(!(argc == 2 || argc == 3)) {
		printf("usage: pco <infile> <outfile>\n\n");
		printf("file format:\n");
		printf("[number of column (n)]\n");
		printf("[element 1,1 of matrix]......[element 1,n]\n");
		printf("          :                        :\n");
		printf("[element n,1 of matrix]......[element n,n]\n");
		return EXIT_FAILURE;
	}

	/* open input file */
	strcpy(buffer, argv[1]);
	fp_in = fopen(buffer, "r");
	if(fp_in == NULL) {
		printf("can't open input file: %s\n", buffer);
		return EXIT_FAILURE;
	}
	
	/* open output file */
	fp_out = NULL;
	if(argc == 3) {
		strcpy(buffer, argv[2]);
		fp_out = fopen(buffer, "w");
		if(fp_out == NULL)
			printf("cannot open output file: %s\n", buffer);
	}

	/* get column number of similarity matrix */
	fscanf(fp_in, "%s", buffer);
	analsize = atoi(buffer);
	
	/* get memory space */
	sim = new_matrix(analsize, analsize);
	evec = new_matrix(analsize, analsize);
	evec_work = new_matrix(analsize, analsize);
	eval = new_vector(analsize);
	eval_work = new_vector(analsize);
	l_mean = new_vector(analsize);
	cont = new_vector(analsize);
	cumcont = new_vector(analsize);
	
	/* get matrix element */
	for(i = 0; i < analsize; i++ ) {
		for(j = 0; j < analsize; j++ ) {
			fscanf(fp_in, "%s", buffer);
			sim[i][j] = atof(buffer);
		}
	}

	/* show similarity matrix */
	printf("[SIMILARITY MATRIX]\n");
	mat_print(sim, analsize, analsize, " %8.4f");
	printf("\n\n");
	if(fp_out != NULL) {
		fprintf(fp_out, "[SIMILARITY MATRIX]\n");
		mat_fprint(fp_out, sim, analsize, analsize, ",%f");
		fprintf(fp_out, "\n\n");
	}
	
	/* prepare for double centering */
	init_vector(l_mean, analsize);
	t_mean = 0;
	for(i = 0; i < analsize; i++ ) {
		for(j = 0; j < analsize; j++ )
			l_mean[i] += sim[i][j];
		t_mean += l_mean[i];
		l_mean[i] /= analsize;
	}
	t_mean /= analsize * analsize;

	/* double centering */
	for(i = 0; i < analsize; i++ )
		for(j = 0; j < analsize; j++ )
			sim[i][j] = sim[i][j] - l_mean[i] - l_mean[j] + t_mean;
	printf("[DOUBLE CENTERING MATRIX]\n");
	mat_print(sim, analsize, analsize, " %8.4f");
	printf("\n\n");
	if(fp_out != NULL) {
		fprintf(fp_out, "[DOUBLE CENTERING MATRIX]\n");
		mat_fprint(fp_out, sim, analsize, analsize, ",%f");
		fprintf(fp_out, "\n\n");
	}

	/* princomp */
	for(i = 0; i < analsize; i++ )
		for(j = 0; j < analsize; j++ )
			evec[i][j] = sim[i][j];
	if(eigen(analsize, evec, eval, eval_work)) {
		for(i = 0; i < analsize; i++ )
			for(j = 0; j < analsize; j++ )
				evec[i][j] = sim[i][j];
		if(jacobi(analsize, evec, eval, evec_work)) {
			free_vector(eval);
			free_vector(eval_work);
			free_matrix(evec);
			free_matrix(sim);
			free_matrix(evec_work);
			free_vector(cont);
			free_vector(cumcont);
			return EXIT_FAILURE;
		}
	}
	
	/* show eigen value */
	sum1 = 0;
	pd = 1;
	for(i = 0; i < analsize; i++ ) {
		if(eval[i] > 0)
			sum1 += eval[i];
		if(eval[i] < - EPSILON) {
			pd = 0;
			break;
		}
	}
	sum2 = 0;
	for(i = 0; i < analsize; i++ ) {
		if(eval[i] > 0)
			sum2 += eval[i];
		if(pd) {
			cont[i] = eval[i] / sum1;
			cumcont[i] = sum2 / sum1;
		} else {
			cont[i] = 0.0;
			cumcont[i] = 0.0;
		}
	}

	printf("[EIGEN VALUE]\n");
	vec_print(eval, analsize, analsize, " %8.4f");
	printf("[CONTRIBUTION]\n");
	vec_print(cont, analsize, analsize, " %8.4f");
	printf("[CUMULATIVE CONTRIBUTION]\n");
	vec_print(cumcont, analsize, analsize, " %8.4f");
	printf("\n\n");
	if(fp_out != NULL) {
		fprintf(fp_out, "[EIGEN VALUE]");
		for(i = 0; i < analsize; i++)
			fprintf(fp_out, ",PCO%d", i + 1);
		fprintf(fp_out, "\n");
		vec_fprint(fp_out, eval, analsize, analsize, ",%f");
		fprintf(fp_out, "[CONTRIBUTION]");
		for(i = 0; i < analsize; i++)
			fprintf(fp_out, ",PCO%d", i + 1);
		fprintf(fp_out, "\n");
		vec_fprint(fp_out, cont, analsize, analsize, ",%f");
		fprintf(fp_out, "[CONTRIBUTION]");
		for(i = 0; i < analsize; i++)
			fprintf(fp_out, ",PCO%d", i + 1);
		fprintf(fp_out, "\n");
		vec_fprint(fp_out, cont, analsize, analsize, ",%f");
		fprintf(fp_out, "\n\n");
	}
	
	/* show eigen vector */
	for(i = 0; i < analsize; i++ )
		for(j = 0; j < analsize; j++ )
			evec_work[i][j] = evec[j][i];
	
	printf("[EIGEN VECTOR]\n");
	mat_print(evec_work, analsize, analsize, " %8.4f");
	printf("\n\n");
	if(fp_out != NULL) {
		fprintf(fp_out, "[EIGEN VECTOR]");
		for(i = 0; i < analsize; i++)
			fprintf(fp_out, ",PCO%d", i + 1);
		fprintf(fp_out, "\n");
		mat_fprint(fp_out, evec_work, analsize, analsize, ",%f");
		fprintf(fp_out, "\n\n");
	}

	/* show PCO score */
	for(i = 0; i < analsize; i++ )
		for(j = 0; j < analsize; j++ )
			if(eval[j] > 0)
				evec_work[i][j] = evec_work[i][j] * sqrt(eval[j]);
			else
				evec_work[i][j] = 0.0;

	printf("[PCO SCORE]\n");
	mat_print(evec_work, analsize, analsize, " %8.4f");
	printf("\n\n");
	if(fp_out != NULL) {
		fprintf(fp_out, "[PCO SCORE]");
		for(i = 0; i < analsize; i++)
			fprintf(fp_out, ",PCO%d", i + 1);
		fprintf(fp_out, "\n");
		mat_fprint(fp_out, evec_work, analsize, analsize, ",%f");
		fprintf(fp_out, "\n\n");
	}

	
	/* free allocated memory */
	free_vector(l_mean);
	free_vector(eval);
	free_vector(eval_work);
	free_matrix(evec);
	free_matrix(sim);
	free_matrix(evec_work);
	free_vector(cont);
	free_vector(cumcont);
	return EXIT_SUCCESS;
}
